/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMCharacterData.idl"

/**
 * The nsIDOMText interface inherits from nsIDOMCharacterData and represents 
 * the textual content (termed character data in XML) of an Element or Attr.
 *
 * For more information on this interface please see 
 * http://dvcs.w3.org/hg/domcore/raw-file/tip/Overview.html
 */

[uuid(67273994-6aff-4091-9de9-b788a249f783)]
interface nsIDOMText : nsIDOMCharacterData
{
  nsIDOMText                      splitText(in unsigned long offset)
                                       raises(DOMException);

  /**
   * The concatenation of all logically adjacent text nodes with this text
   * node, where "logically adjacent" consists of all text nodes which can be
   * reached by traversing the document tree in either direction without
   * passing an element, comment, or processing-instruction boundary.  
   */
  readonly attribute DOMString wholeText;
};
